/**********************************************************************//**
* @file		AC30M1x164_hal_adc.h
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library on AC30M1x164
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


#ifndef _AC30M1x164_ADC_H_
#define _AC30M1x164_ADC_H_

/* Includes ------------------------------------------------------------------- */
#include "AC30M1x64.h"
#include "AC30M1x64_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif

/* Private macros ------------------------------------------------------------- */

/* -------------------------- BIT DEFINITIONS --------------------------------- */
/**********************************************************************
 * Macro defines for ADC  control register
 **********************************************************************/
/**  ADC convert in power down mode */
#define ADC_CCR_PDN			((1UL<<7))

/**********************************************************************
 * Macro defines for ADC control2 register
 **********************************************************************/
/**  Start mask bits */
#define ADC_CR_START_MASK	((1UL<<0))
/**  Stop mask bits */
#define ADC_CR_STOP_MASK	((1UL<<7))


/**********************************************************************
 * Macro defines for ADC Status register
**********************************************************************/
#define ADC_STAT_END							((1UL<<7))
#define ADC_STAT_BUSY						((1UL<<6))
#define ADC_STAT_DMAOVERRUN			((1UL<<5))

#define ADC_STAT_DMA							((1UL<<4))
#define ADC_STAT_TRIGGER					((1UL<<3))
#define ADC_STAT_SEQ							((1UL<<2))
#define ADC_STAT_SINGLE						((1UL<<0))


/**********************************************************************
 * Macro defines for ADC Interrupt register
 **********************************************************************/
/** These bits allow control over which A/D channels generate
 * interrupts for conversion completion */
#define ADC_INTEN_DMA					((1UL<<4))
#define ADC_INTEN_TRIGGER			((1UL<<3))
#define ADC_INTEN_SEQ					((1UL<<2))
#define ADC_INTEN_SINGLE				((1UL<<0))

/**********************************************************************
 * Macro defines for ADC Data register
 **********************************************************************/
/** When DONE is 1, this field contains result value of ADC conversion */
#define ADC_DR_RESULT(n)			((n)&0xFFF0)
/** When DONE is 1, this field contains result channel value of ADC conversion */
#define ADC_GET_CH_RESULT(n)	((n)&0xF)


/* Public Types --------------------------------------------------------------- */
#define ADC_SINGLE_MODE                  0  /*!< adc single conversion mode */
#define ADC_BURST_MODE                  1  /*!< adc burst conversion mode */

#define ADC_INTERNAL_CLK                  0  /*!< adc single conversion mode */
#define ADC_EXTERNAL_CLK                 1  /*!< adc continuous conversion mode */

#define ADC_TRIGGER_DISABLE           0  /*!< Event Trigger Disabled/Soft-Trigger Only */
#define ADC_TRIGGER_TIMER               1  /*!< Timer Event Trigger */
#define ADC_TRIGGER_MPWM0            2  /*!< MPWM0 Event Trigger */

typedef struct {
/* MR */	
	uint32_t Mode;		/**< Mode, should be:
														- ADC_SINGLE_MODE
														- ADC_CONTINUOUS_MODE
														- ADC_BURST_MODE */
	uint32_t DmaOpt;					/**< DmaOpt = ENABLE or DISABLE */
	uint32_t SampingTime;	
	uint32_t SeqCnt;					/**< SeqCnt = count value 2~8	*/
	uint32_t RestartEn;				/**< RestartEn = ENABLE or DISABLE */			
	uint32_t TrgSel;	
	
/* CR1 */
	uint32_t UseClk;
	uint32_t InClkDiv;
} ADC_CFG_Type;



/* Public Functions ----------------------------------------------------------- */
/* Init/DeInit ADC peripheral ----------------*/

HAL_Status_Type HAL_ADC_Init(ADC_Type *ADCx, ADC_CFG_Type *ADC_ConfigStruct);
HAL_Status_Type  HAL_ADC_DeInit(ADC_Type *ADCx);

/* Enable/Disable ADC functions --------------*/
HAL_Status_Type  HAL_ADC_PowerdownCmd(ADC_Type *ADCx, FunctionalState NewState);

HAL_Status_Type  HAL_ADC_ChannelSel(ADC_Type *ADCx, uint32_t Channel);
HAL_Status_Type  HAL_ADC_TriggerlSel(ADC_Type *ADCx, uint32_t Trigger);

HAL_Status_Type  HAL_ADC_Start(ADC_Type *ADCx);
HAL_Status_Type  HAL_ADC_Stop(ADC_Type *ADCx);

/* Configure ADC functions -------------------*/
HAL_Status_Type  HAL_ADC_EdgeStartConfig(ADC_Type *ADCx, uint8_t EdgeOption);
HAL_Status_Type  HAL_ADC_ConfigInterrupt(ADC_Type *ADCx, uint8_t IntType, FunctionalState NewState);

/* Get ADC information functions -------------------*/
uint16_t HAL_ADC_GetData(ADC_Type *ADCx);

/* Get/Clear ADC status  -------------------*/
uint32_t HAL_ADC_GetStatus(ADC_Type *ADCx);
HAL_Status_Type  HAL_ADC_ClearStatus(ADC_Type *ADCx, uint32_t status);

#ifdef __cplusplus
}
#endif


#endif /* _AC30M1x164_ADC_H_ */

/* --------------------------------- End Of File ------------------------------ */
