/**********************************************************************//**
* @file		AC30M1x164_hal_adc.c
* @brief	Contains all functions support for ADC firmware library on AC30M1x164
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


/* Includes ----------------------------------------------------------------- */
#include "AC30M1x64_hal_adc.h"
#include "AC30M1x64_hal_scu.h"



/* Public Functions ----------------------------------------------------------- */
/**********************************************************************
 * @brief 		Initial for ADC
 * @param[in]	ADCx pointer to ADC_Type, should be: ADCx
 * @param[in]	ADC_ConfigStruct
 * @return 		None
 **********************************************************************/
HAL_Status_Type HAL_ADC_Init(ADC_Type *ADCx, ADC_CFG_Type *ADC_ConfigStruct)
{

	uint32_t tempreg;
	
	// Turn on power and clock
	if(ADCx == AD)
	{
		SCU->PER2&=~(1<<20);
		SCU->PCER2&=~(1<<20);
		/* Set up peripheral clock for ADC module */
		SCU->PER2|=(1<<20);
		SCU->PCER2|=(1<<20);
	}
    //-------------You must change mode to ADC single-mode before you start Initialize ADC -----------------
    ADCx->MR = ((1<<12)|(1<<7)); //, STSEL, ADCEN
    ADCx->CCR = (4<<8);
    ADCx->CR = (1);
	while((ADCx->SR&0x40)==0); 	//wait until End-of-Conversion
	while((ADCx->SR&0x80)==0); 	//wait until End-of Conversion
	ADCx->SR = 0xFF;		// Clear all flag
    ADCx->MR = (0);	//init MR
	//--------------------End of changeof single modet---------------------------------------------------
	
	tempreg=0;
	switch(ADC_ConfigStruct->Mode){
		case ADC_SINGLE_MODE:
			tempreg = 0
				| ((ADC_ConfigStruct->SampingTime & 0x1f)<<12)  // STSEL 
				| (((ADC_ConfigStruct->SeqCnt-1) & 7)<<8)  // SEQCNT
				| (1<<7)  // ADCEN
				| (((ADC_ConfigStruct->RestartEn) & 1)<<6)  // ARST			
				| (0<<4)  // ADCMOD
				| (((ADC_ConfigStruct->TrgSel) & 3)<<0)  // TRGSRC		
				;				
			break;
		case ADC_BURST_MODE:
			tempreg = 0
				| ((ADC_ConfigStruct->SampingTime & 0x1f)<<12)  // STSEL 
				| (((ADC_ConfigStruct->SeqCnt-1) & 7)<<8)  // SEQCNT
				| (1<<7)  // ADCEN
				| (((ADC_ConfigStruct->RestartEn) & 1)<<6)  // ARST	
				| (1<<4)  // ADCMOD
				| (((ADC_ConfigStruct->TrgSel) & 3)<<0)  // TRGSRC	
				;
			break;
		default:
			break;
	}		
	ADCx->MR = tempreg;


	tempreg = 0
		| (0<<8)  // CKDIV  1.0Mhz * 15 =15Mhz
		| (0<<7)  // ADCPD
		| (ADC_ConfigStruct->UseClk<<6)  // EXTCLK
		| (0<<5)  // CLKINVT		
		;
	if (ADC_ConfigStruct->UseClk == ADC_INTERNAL_CLK) {
		tempreg |= ((ADC_ConfigStruct->InClkDiv & 0x7f)<<8)  // CKDIV  1.0Mhz * 15 =15Mhz,  PCLK / CLKDIV =8Mhz / 1 = 8Mhz 
		;		
	}
	
	ADCx->CCR = tempreg;	
	ADCx->CSCR = 0;
	
	HAL_ADC_Stop(ADCx);
	
	return HAL_OK;

}


/**********************************************************************//**
* @brief 		Close ADC
* @param[in]	ADCx pointer to ADC_Type, should be: ADCx
* @return 		None
**********************************************************************/
HAL_Status_Type HAL_ADC_DeInit(ADC_Type *ADCx)
{
	ADCx->CR = (1<<7); // stop
	ADCx->CCR = 0;
	ADCx->IER = 0;
	ADCx->MR = 0;	
	
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Get Result conversion from A/D data register
* @param[in]	ADCx pointer to ADC_Type, should be: ADCx
* @return 		Result of conversion
**********************************************************************/
uint16_t HAL_ADC_GetData(ADC_Type *ADCx)
{
	uint16_t adc_value;

	adc_value = ADCx->DR0;
	return ADC_DR_RESULT(adc_value);
}

/**********************************************************************//**
* @brief 		Set start mode for ADC
* @param[in]	ADCx pointer to ADC_Type, should be: ADCx
* @return 		None
*********************************************************************/
HAL_Status_Type HAL_ADC_Start(ADC_Type *ADCx)
{
	ADCx->CR =ADC_CR_START_MASK;
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Set stop mode for ADC
* @param[in]	ADCx pointer to ADC_Type, should be: ADCx
* @return 		None
*********************************************************************/
HAL_Status_Type HAL_ADC_Stop(ADC_Type *ADCx)
{
	ADCx->CR =ADC_CR_STOP_MASK;
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Set AD conversion in power mode
* @param[in]	ADCx pointer to ADC_Type, should be: ADCx
* @param[in]	NewState
* 					- DISABLE : AD converter is normal mode
* 					- ENABLE: AD Converter is in power down mode
* @return 		None
**********************************************************************/
HAL_Status_Type HAL_ADC_PowerdownCmd(ADC_Type *ADCx, FunctionalState NewState)
{
	ADCx->CCR &= ~ADC_CCR_PDN;
	if (NewState){
		ADCx->CCR |= ADC_CCR_PDN;
	}
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		ADC interrupt configuration
* @param[in]	ADCx pointer to ADC_Type, should be: ADCx
* @param[in]	IntType: type of interrupt, should be:
* 					- ADC_INTEN_TRIGGER
* 					- ADC_INTEN_SEQ
* 					- ADC_INTEN_SINGLE
* @param[in]	NewState:
* 					- SET : enable ADC interrupt
* 					- RESET: disable ADC interrupt
* @return 		None
**********************************************************************/
HAL_Status_Type HAL_ADC_ConfigInterrupt(ADC_Type *ADCx, uint8_t IntType, FunctionalState NewState)
{
	ADCx->IER &= ~IntType;
	if (NewState){
		ADCx->IER |= IntType;
	}
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Select ADC channel number
* @param[in]	ADCx pointer to ADC_Type, should be: ADCx
* @param[in]	Channel channel number
* @return 		None
**********************************************************************/
HAL_Status_Type HAL_ADC_ChannelSel(ADC_Type *ADCx, uint32_t Channel)
{
	ADCx->SCSR = Channel;
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Select Burst ADC channel number
* @param[in]	ADCx pointer to ADC_Type, should be: ADCx
* @param[in]	Channel channel number
* @return 		None
**********************************************************************/
HAL_Status_Type  HAL_ADC_TriggerlSel(ADC_Type *ADCx, uint32_t Trigger)
{
	ADCx->TRG = Trigger;
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Get ADC channel status 
* @param[in]	ADCx pointer to ADC_Type, should be: ADCx
* @return 		ADC status register
**********************************************************************/
uint32_t HAL_ADC_GetStatus(ADC_Type *ADCx)
{
	return (ADCx->SR);
}

/**********************************************************************//**
* @brief 		Clear ADC Chanel status
* @param[in]	ADCx pointer to ADC_Type, should be: ADCx
* @param[in]	status  channel number
* @return 		None
**********************************************************************/
HAL_Status_Type HAL_ADC_ClearStatus(ADC_Type *ADCx, uint32_t status)
{
	ADCx->SR=status;
	return HAL_OK;
}



/* --------------------------------- End Of File ------------------------------ */

