/**********************************************************************//**
* @file		AC30M1x164_hal_frt.c
* @brief	Contains all functions support for FRT(Free Running TImer) firmware library on AC30M1x164
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

/* Includes ---------------------------------------------------------------- */
#include "AC30M1x64_hal_frt.h"
#include "AC30M1x64_hal_libcfg.h"
#include "AC30M1x64_debug_frmwrk.h"	// for Displaying Results at UART Terminal

/* Public Functions ----------------------------------------------------------- */
/*********************************************************************//**
 * @brief		Initialization register related FRT function at SCU
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_FRT_ModeInit(FRT_Type *FRTx)
{	
	if(FRTx == FRT)
	{		
		SCU->PER1 &= ~(1 << 26); 
		SCU->PCER1 &= ~(1 << 26);
		
		SCU->PER1 |= (1 <<26);			// FRT Enable
		SCU->PCER1 |= (1 << 26);			// FRT Clock Enable
	}

	HAL_FRT_SetPeriod(FRTx, 0);	// Set Period
	
	/* Mode Register Reset */
	FRTx->MR = 0x00;
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		Set Clock Source Type of FRT
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @param[in]	Clock Source:
 *					- FRT_LSIOSC : Low speed Internal Clock
 *					- FRT_EXTOSC : External Oscillator Clock Devided by 32
 *					- FRT_SUBOSC : Sub Oscillator Clock
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_FRT_ClockSource(FRT_Type *FRTx, uint8_t ClkSource)
{
	uint32_t regtemp;
	
	regtemp = FRTx->MR;
	regtemp &= ~(3<<4);
	regtemp |= (ClkSource << 4);
	FRTx->MR = regtemp;		// Set FRT Clock Source
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		Get Clock Source Type of FRT
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @return 		Clock Source Type
 **********************************************************************/
uint8_t HAL_FRT_GetClockSource(FRT_Type *FRTx)
{
	uint8_t ClkSource;
	
	ClkSource = (FRTx->MR & 0x30);

/*	
	if((ClkSource & 0x30) == 0x00)				// FRT_LSIOSC
		_DBG("Low Speed Internal Oscillator Clock\r\n");
	else if((ClkSource & 0x30) == 0x10)			// FRT_EXTOSC
		_DBG("Sub Oscillator Clock\r\n");
	else if((ClkSource & 0x30) == 0x20)			// FRT_SUBOSC
		_DBG("External Oscillator Clock\r\n");
	else 
		_DBG("Reserved\r\n");
*/
	
	return ClkSource;
}

/*********************************************************************//**
 * @brief		Set Period Value of FRT
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @param[in]	FRT Channel Pointer,
 *				SetPeriod : Unsigned Int 32
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_FRT_SetPeriod(FRT_Type *FRTx, uint32_t Period)
{
	FRTx->PER = Period;							// Set Period
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		Get Period Value of FRT
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @param[in]	FRT Channel Pointer
 * @return 		Period
 **********************************************************************/
uint32_t HAL_FRT_GetPeriod(FRT_Type *FRTx)
{			
	return FRTx->PER;							// Get FRT Period Register Value
	
}

/*********************************************************************//**
 * @brief		Set FRT Operation 
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @param[in]	isset	Enable or Disable
 *					- 1		:Enable 
 *					- 0		:Disable
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_FRT_Run(FRT_Type *FRTx, uint8_t isset)
{
	if(isset)
	{
		FRTx->CR |= 0x01;		// FRT Enable
//		_DBG("FRT is Enabled.\r\n");
	}
	else
	{
		FRTx->CR &= ~0x01;		// FRT Disable				
//		_DBG("FRT is Disabled.\r\n");
	}
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		Set FRT Match Interrupt
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @param[in]	isset	Enable or Disable
 *					- 1		:Enable 
 *					- 0		:Disable
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_FRT_MatchInterrupt(FRT_Type *FRTx, uint8_t isset)
{
	if(isset)
	{
		FRTx->MR |= (0x01);		// FRT Match Interrupt Enable					
//		_DBG("FRT Interrupt is Enabled.\r\n");		
	}
	else
	{
		FRTx->MR &= ~(0x01);		// FRT Match Interrupt Disable				
//			_DBG("FRT Interrupt is Disabled.\r\n");
	}
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		Set FRT OVERFLOW Interrupt Mode
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @param[in]	isset	Enable or Disable
 *					- 1		:Enable 
 *					- 0		:Disable
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_FRT_OverflowInterrupt(FRT_Type *FRTx, uint8_t isset)
{
	if(isset)		
	{
		FRTx->MR |= (0x02);		// FRT Overflow Interrupt Enable
//		_DBG("FRT Overflow Interrupt is Enabled.\r\n");
	}
	else
	{
		FRTx->MR &= ~(0x02);		// FRT Overflow Interrupt Disable
//		_DBG("FRT Overflow Interrupt is Disabled.\r\n");
	}
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		Set FRT Match Clear Disable Bit
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @param[in]	isset	Enable or Disable
 *					- 1		:Enable 
 *					- 0		:Disable
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_FRT_CounterMatchClearDisableBit(FRT_Type *FRTx, uint8_t isset)
{
	if(isset)
	{
		FRTx->MR |= (0x04);		// FRT Counter Match Clear Disable bit Disable
//		_DBG("FRT Counter Match Clock Bit Disabled.\r\n");		
	}
	else
	{
		FRTx->MR &= ~(0x04);		// FRT Counter Match Clear Disable bit Enable
//		_DBG("FRT Counter Match Clock Bit Enabled.\r\n");
	}
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		Set Status of FRT
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @param[in]	StatusType	Status Type
 *					- FRT_MATCH_FLAG
					- FRT_OVERFLOW_FLAG
					- FRT_READACK_FLAG
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_FRT_SetStatus(FRT_Type *FRTx, uint8_t StatusType)
{
	FRTx->SR = StatusType;
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		Get Status of FRT
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @return 		8-bit Status
 **********************************************************************/
uint8_t HAL_FRT_GetStatus(FRT_Type *FRTx)
{	
	return FRTx->SR;			// Get Status Register Value
}

/*********************************************************************//**
 * @brief		Set Counter of FRT
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @param[in]	isRead	FRT Counter Read Request Bit
					- 0		:not Read
					- 1		:Read
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_FRT_Counter(FRT_Type *FRTx, uint8_t isRead)
{
	if(isRead)
	{
		FRTx->CR |= (1 << 3);	// Set FRT Counter Read Request Bit
//		_DBG("Ready FRT Counter Set.\r\n");
	}
	else
	{
		FRTx->CR &= ~(1 << 3);	// Unset FRT Counter Read Request Bit
//		_DBG("End FRT Counter.\r\n");			
	}
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		Get Counter Value of FRT
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @return 		32-bit Count Value (uint32_t)
 **********************************************************************/
uint32_t HAL_FRT_GetCounterVal(FRT_Type *FRTx)
{
	uint32_t CountValue = 0;

	CountValue = FRTx->CNT;				// Get FRTx Counter Value

	return CountValue;
}

/*********************************************************************//**
 * @brief		Clear Counter Value of FRT
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_FRT_ClearCounter(FRT_Type *FRTx)
{
	uint32_t regtemp;
	
	regtemp = FRTx->CR ;
	regtemp |= (1<<2); 
	FRTx->CR = regtemp;			// Clear FRTx Counter Register Value
//	_DBG("FRTx Counter Value is Cleard.\r\n");
	return HAL_OK;

}

/*********************************************************************//**
 * @brief		Set Counter Hold of FRT
 * @param[in]	FRTx	Pointer to selected FRT peripheral, should be:
 * 					- FRT	:FRT peripheral
 * @param[in]	isRead	FRT Counter Hold Bit
					- 0		:NOT Hold
					- 1		:Hold
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_FRT_HoldCounter(FRT_Type *FRTx, uint8_t isHold)
{
	if(isHold)
	{
		FRTx->CR |= 0x02;					// FRTx Counter Hold
//		_DBG("Hold FRTx Counter.\r\n");
	}
	else
	{
		FRTx->CR &= ~(0x02);				// FRTx Counter Release
//		_DBG("Release FRTx Counter.\r\n");
	}
	return HAL_OK;
}
/* --------------------------------- End Of File ------------------------------ */
