/**********************************************************************//**
* @file		AC30M1x164_hal_mpwm.c
* @brief	Contains all functions support for MPWM firmware library on AC30M1x164
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


/* Includes -------------------------------------------------------------- */
#include "AC30M1x64_hal_scu.h"
#include "AC30M1x64_hal_mpwm.h"
#include "AC30M1x64_hal_libcfg.h"



/* Public Functions -------------------------------------------------------- */
/* MPWM Init/DeInit functions -----------------------------------------------*/
/********************************************************************//**
 * @brief		Initializes the MPWMx peripheral according to the specified
 *               parameters.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @return 		None
 *********************************************************************/
HAL_Status_Type  HAL_MPWM_Init(MPWM_Type *MPWMx)
{
	if(MPWMx == MP)
	{
		SCU->PER2&=~(1<<16);
		SCU->PCER2&=~(1<<16);
		
		/* Set up peripheral clock for MPWM0 module */
		SCU->PER2|=(1<<16);
		SCU->PCER2|=(1<<16);
	}
	
// PWMEN bit = '1'
	MPWMx->CR1 = 0
		|(0<<8)                    // IRQN[2:0]
		|(1<<0)                    // PWMEN
		;

	MPWMx->CR2 = 0;         // PSTART disable 
	MPWMx->OLR = 0;         // MPWM Output Level 
	MPWMx->FOLR = 0;       // MPWM Force Output Level
	MPWMx->PCR0 = 0;        // Protection 0 Control 
	MPWMx->PSR0 = (0xca<<8) | 0;        // Protection 0 Status  
	MPWMx->PCR1 = 0;        // Protection 1 Control 
	MPWMx->PSR1 = (0xca<<8) | 0;        // Protection 1 Status  
	MPWMx->ATR1 = 0;
	MPWMx->ATR2 = 0;
	MPWMx->ATR3 = 0;
	MPWMx->ATR4 = 0;
	MPWMx->ATR5 = 0;
	MPWMx->ATR6 = 0;	
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		De-initializes the MPWMx peripheral registers to their
 *              default reset values.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_MPWM_DeInit(MPWM_Type* MPWMx)
{
	MPWMx->CR1 = 0; // disable PWMEN
		
	if (MPWMx == MP)
	{
		/* Set up peripheral clock for UART0 module */
		SCU->PER2&=~(1<<16);
		SCU->PCER2&=~(1<<16);
	}
	return HAL_OK;
}

/*********************************************************************//**
 * @brief		Enable or Disable PWM start
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	NewState New State of MPWMx peripheral's operation, should be:
 * 					- ENABLE
 * 					- DISABLE
 * @return none
 **********************************************************************/
HAL_Status_Type  HAL_MPWM_Start(MPWM_Type* MPWMx, FunctionalState NewState)
{
	if (NewState == ENABLE) {
		MPWMx->CR2 = (1<<0);
	}
	else 	{
		MPWMx->CR2 = (0<<0);
	}
	return HAL_OK;
}

/**********************************************************************//**
 * @brief		Set PWM Output Control data
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	Data	Data to control
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_MPWM_SetOutput(MPWM_Type* MPWMx, uint32_t Data)
{
	MPWMx->OLR =(Data&0x3f);
//	MPWMx->OLR = 0;
//		|(0<<5)   // WHL      
//		|(0<<4)   // VHL 
//		|(0<<3)   // UHL 
//		|(0<<2)   // WLL 
//		|(0<<1)   // VLL 
//		|(0<<0)   // ULL 	
//		;	
	return HAL_OK;

}

/**********************************************************************//**
 * @brief		Set PWM Force Output Control data
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	Data	Data to control
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_MPWM_SetFOutput(MPWM_Type* MPWMx, uint32_t Data)
{
	MPWMx->FOLR =(Data&0x3f);
//	MPWMx->FOLR = 0
//		|(0<<5)   // WHFL       
//		|(0<<4)   // VHFL 
//		|(0<<3)   // UHFL 
//		|(0<<2)   // WLFL 
//		|(0<<1)   // VLFL 
//		|(0<<0)   // ULFL 	
//		;		
	return HAL_OK;

}

/**********************************************************************//**
 * @brief		Set PWM Protection Control data
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	Data	Data to control
 * @param[in]	ch	channel
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_MPWM_SetProtCtrl(MPWM_Type* MPWMx, uint32_t Data, uint32_t ch)
{
	if (ch==0){
		MPWMx->PCR0 =(Data&0xffff);        // Protection 0 Control 		
	}
	else if (ch==1){
		MPWMx->PCR1 = (Data&0xffff);        // Protection 1 Control 		
	}
	return HAL_OK;
}

/**********************************************************************//**
 * @brief		Set PWM Protection Status data
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	Data	Data to control
 * @param[in]	ch	channel
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_MPWM_SetProtStat(MPWM_Type* MPWMx, uint32_t Data, uint32_t ch)
{
	if (ch==0){
		MPWMx->PSR0 = (0xca<<8) | (Data&0xff);        // Protection 0 Status  		
	}
	else if (ch==1){	
		MPWMx->PSR1 = (0xac<<8) | (Data&0xff);        // Protection 1 Status  			
	} 	
		return HAL_OK;
}

/********************************************************************//**
 * @brief 		Enable or disable specified MPWM interrupt.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	MPIntCfg	Specifies the interrupt src,
 * @param[in]	NewState New state of specified MPWM interrupt type,
 * 				should be:
 * 					- ENALBE	:Enable this MPWM interrupt type.
 * 					- DISALBE	:Disable this MPWM interrupt type.
 * @return 		None
 *********************************************************************/
HAL_Status_Type  HAL_MPWM_ConfigInterrupt(MPWM_Type* MPWMx, uint32_t MPIntCfg, FunctionalState NewState)
{
	if (NewState == ENABLE)
	{
		MPWMx->IER |= MPIntCfg;
	}
	else
	{
		MPWMx->IER &= (~MPIntCfg) & 0xff;
	}
/*
	MPx->IER = 0
	|(0<<7)   // PRDIEN
	|(0<<6)   // BOTIEN
	|(0<<5)   // WHIE    
	|(0<<4)   // VHIE 
	|(0<<3)   // UHIE 
	|(0<<2)   // WLIE 
	|(0<<1)   // VLIE 
	|(0<<0)   // ULIE 	
		;
*/	
	return HAL_OK;

}

/********************************************************************//**
 * @brief 		Configure PWM Mode setting.
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	update			0:duty and period after match, 1:after 2 PWM clocks
 * @param[in]	tup				0:every period match, 1:every period 	
 * @param[in]	bup				0:every period match, 1:e bottom match	
 * @param[in]	mchmode	0:2-ch symmetric, 1:1-ch asymmetric, 2:1-ch symmetric
 * @param[in]	updown		0: PWM UP count, 1:PWM UP/DOWN	 									
 * @return 		None
 *********************************************************************/
HAL_Status_Type  HAL_MPWM_Cmd(MPWM_Type* MPWMx,uint32_t motorb, uint32_t update, uint32_t tup, uint32_t bup, uint32_t mchmode, uint32_t updown)
{
	MPWMx->MR = 0
		|((motorb&0x01)<<15)				// MOTORB    0:motor mode, 1:normal mode	
		|((update&0x01)<<7)				// UPDATE    0:duty and period after match, 1:after 2 PWM clocks
		|((tup&0x01)<<5)						// TUP        0:every period match, 1:every period match		
		|((bup&0x01)<<4)						// BUP        0:every period match, 1:every bottom match	
		|((mchmode&0x03)<<1)			// MCHMOD  0:2-ch symmetric, 1:1-ch asymmetric, 2:1-ch symmetric
		|((updown)<<0)						// UPDOWN  0: PWM UP count, 1:PWM UP/DOWN 
		;	
	return HAL_OK;
}


/********************************************************************//**
 * @brief 		Set period data .
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	period			MPWM period data									
 * @return 		None
 *********************************************************************/
HAL_Status_Type  HAL_MPWM_SetPeriod(MPWM_Type* MPWMx, uint32_t period)
{
	MPWMx->PRD = (period&0xffff);  // period. it sould be larger than 0x10	
	return HAL_OK;
}

/********************************************************************//**
 * @brief 		Set duty U high/low data .
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	dutyH			MPWM UdutyH data
 * @param[in]	dutyL			MPWM UdutyL data			 							
 * @return 		None
 *********************************************************************/
HAL_Status_Type  HAL_MPWM_SetUDuty(MPWM_Type* MPWMx, uint32_t udutyH, uint32_t udutyL)
{
	MPWMx->DUH = ((udutyH) &0xffff);  // if using I/O control function, set period data	
	MPWMx->DUL = ((udutyL) &0xffff); 	
	return HAL_OK;
}

/********************************************************************//**
 * @brief 		Set duty V high/low data .
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	vdutyH			MPWM VdutyH data
 * @param[in]	vdutyL			MPWM VdutyL data			 							
 * @return 		None
 *********************************************************************/
HAL_Status_Type  HAL_MPWM_SetVDuty(MPWM_Type* MPWMx, uint32_t vdutyH, uint32_t vdutyL)
{
	MPWMx->DVH = ((vdutyH) &0xffff);  // if using I/O control function, set period data	
	MPWMx->DVL = ((vdutyL) &0xffff); 	
	return HAL_OK;
}

/********************************************************************
 * @brief 		Set duty W high/low data .
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	wdutyH			MPWM WdutyH data
 * @param[in]	wdutyL			MPWM WdutyL data			 							
 * @return 		None
 *********************************************************************/
HAL_Status_Type  HAL_MPWM_SetWDuty(MPWM_Type* MPWMx, uint32_t wdutyH, uint32_t wdutyL)
{
	MPWMx->DWH = ((wdutyH) &0xffff);  // if using I/O control function, set period data	
	MPWMx->DWL = ((wdutyL) &0xffff); 	
	return HAL_OK;
}

/********************************************************************//**
 * @brief 		Set dead time
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	dten			dead time enable 
 * @param[in]	pshrt		Protect short condition 
 * @param[in]	clk			0:PWM CLK /4, 1:PWM CLK /16 
 * @param[in]	clkdata		0x01~0xff dead time value									
 * @return 		None
 *********************************************************************/
HAL_Status_Type  HAL_MPWM_SetDeadTime(MPWM_Type* MPWMx, uint32_t dten, uint32_t pshrt, uint32_t clk, uint32_t clkdata)
{
 	MPWMx->DTR = 0
		|((dten&0x01)<<15)                  // DTEN
		|((pshrt&0x01)<<14)                  // PSHRT	
		|((clk&0x01)<<8)                    // DTCLK
		|((clkdata&0xff)<<0)                    // DT
		; 	
	return HAL_OK;
}

/**********************************************************************//**
 * @brief		Set PWM ADC Trigger Counter
 * @param[in]	MPWMx	MPWM peripheral selected, should be:
 * 					- MPx	:MPWM0 peripheral
 * @param[in]	atudt	Trigger register update mode
 * @param[in]	atmod	ADC trigger Mode register
 * @param[in]	atcnt	ADC Trigger counter
 * @param[in]	ch	channel
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_MPWM_SetATR(MPWM_Type* MPWMx, uint32_t atudt, uint32_t atmod, uint32_t atcnt, uint32_t ch)
{
	uint32_t tempreg;
	
	tempreg = 0
		|((atudt&0x01)<<19)
		|((atmod&0x03)<<16)
		|((atcnt&0xFFFF)<<0)	
		;
	if (ch==1){
		MPWMx->ATR1 = tempreg; 		
	}
	else if (ch==2){	
		MPWMx->ATR2 = tempreg; 				
	}
	else if (ch==3){	
		MPWMx->ATR3 = tempreg; 				
	} 
	else if (ch==4){	
		MPWMx->ATR4 = tempreg; 				
	} 
	else if (ch==5){	
		MPWMx->ATR5 = tempreg; 				
	} 
	else if (ch==6){	
		MPWMx->ATR6 = tempreg; 				
	} 
	return HAL_OK;
}




/* --------------------------------- End Of File ------------------------------ */

