/**********************************************************************//**
* @file		AC30M1x64_hal_pcu.c
* @brief	Contains all functions support for PCU firmware library on AC30M1x64
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


/* Includes ------------------------------------------------------------- */
#include "AC30M1x64_hal_pcu.h"

/**********************************************************************//**
 * @brief 		Configure pin function
 * @param[in]	PCx	Pointer to selected PCU peripheral, should be: A,B,C,D
 * @param[in]	pin_no		Pin number, should be: 0..15
 * @param[in]	func 	Function mode, should be:
 * 					- FUNC0		:Function 0
 * 					- FUNC1		:Function 1
 * 					- FUNC2		:Function 2
 * 					- FUNC3		:Function 3
 * @return		None
 **********************************************************************/
void HAL_GPIO_ConfigFunction(PCU_Type *PCx, uint8_t pin_no, uint32_t func)
{
	uint8_t pin_offset; 
	uint32_t reg_val; 

	//------------------------------------------------------------------------------
	// pin_offset = pin_no * 2
	//------------------------------------------------------------------------------
	pin_offset = (pin_no << 1); 

	//------------------------------------------------------------------------------
	// MR
	//------------------------------------------------------------------------------
 	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register	
	reg_val = PCx->MR;
	reg_val &= ~(PCU_MR_FUNC_Msk << pin_offset); 
	reg_val |= (func << pin_offset); 

	PCx->MR=reg_val;
	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 		
}

/**********************************************************************//**
 * @brief 		Configure pin mode
 * @param[in]	PCx	Pointer to selected PCU peripheral, should be: A,B,C,D
 * @param[in]	pin_no		Pin number, should be: 0..15
 * @param[in]	dir_type	Pin mode, should be:
 * 					- PUSH_PULL_OUTPUT   :0
 * 					- OPEN_DRAIN_OUTPUT :1
 * 					- INPUT                           :2
 * 					- ANALOG                       :3
 * @return		None
 **********************************************************************/
void HAL_GPIO_ConfigOutput(PCU_Type *PCx, uint8_t pin_no, PCU_PORT_MODE dir_type)
{
	uint8_t pin_offset; 
	uint32_t reg_val; 

	//------------------------------------------------------------------------------
	// pin_offset = pin_no * 2
	//------------------------------------------------------------------------------
	pin_offset = (pin_no << 1); 

	//------------------------------------------------------------------------------
	// CR
	//------------------------------------------------------------------------------
 	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register	
	reg_val = PCx->CR; 
	reg_val &= ~(PCU_CR_MODE_Msk << pin_offset); 
	reg_val |= (dir_type << pin_offset); 
	PCx->CR=reg_val; 
	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 		
}

/**********************************************************************//**
 * @brief 		Configure pin pullup
 * @param[in]	PCx	Pointer to selected PCU peripheral, should be: A,B,C,D
 * @param[in]	pin_no		Pin number, should be: 0..15
 * @param[in]	pullup pullup state, should be:
 * 					- DISABLE		:0
 * 					- ENABLE 		:1
 * @return		None
 **********************************************************************/
void HAL_GPIO_ConfigPullup (PCU_Type *PCx, uint8_t pin_no, FunctionalState pullup)
{
	uint32_t	reg_val; 

	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register
	reg_val = PCx->PCR; 
	reg_val &= ~(1<<pin_no); 
	reg_val |= (pullup<<pin_no); 
	PCx->PCR=reg_val; 
	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 		
}

/**********************************************************************//**
 * @brief		Enable GPIO interrupt
 * @param[in]	PCx	Pointer to selected PCU peripheral, should be: A,B,C,D
 * @param[in]	pin_no		Pin number, should be: 0..15
 * @param[in]	pin_en 	Value that contains all bits on GPIO to enable,
 * 					- IER_DISABLE = 0
 *						- IER_LEVEL = 1
 *						- IER_EDGE = 3				
 * @param[in]	int_mode		interrupt mode, should be:
 * 					- ICR_PROHIBIT_INT = 0
 *                     if level
 * 					- ICR_LOW_LEVEL_INT = 1
 * 					- ICR_HIGH_LEVEL_INT = 2
 *                     if level
 * 					- ICR_FALLING_EDGE_INT = 1
 * 					- ICR_RISING_EDGE_INT = 2
 * 					- ICR_BOTH_EDGE_INT = 3
 * @return		None
 **********************************************************************/
void HAL_GPIO_EXTI_Config(PCU_Type *PCx, uint8_t pin_no, uint8_t pin_en, uint8_t int_mode)
{
	uint32_t temp_reg;

	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register	
	// port n interrupt enable: disable or level or edge
	temp_reg = PCx->IER & (uint32_t)(~(3 << (pin_no<<1)));
	temp_reg |= (uint32_t)(pin_en <<  (pin_no<<1));	
	PCx->IER = temp_reg;

	switch (pin_en){
		case 0: // disable 
			PCx->ICR = PCx->ICR & (uint32_t)(~(3 <<  (pin_no<<1)));
			break;			
		case 1: // level
		case 3: // edge			
			temp_reg = PCx->ICR & (uint32_t)(~(3 <<  (pin_no<<1)));
			temp_reg |= (uint32_t)(int_mode <<  (pin_no<<1));		
			PCx->ICR = temp_reg;
			break;			
		default:
			break;
	}
	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 		
}

/**********************************************************************//**
 * @brief		Get GPIO Interrupt Status
 * @param[in]	PCx	Pointer to selected PCU peripheral, should be: A,B,C,D
 * @return		ISR register value 
 **********************************************************************/
uint32_t HAL_GPIO_GetInterruptStatus(PCU_Type *PCx)
{
	return (PCx->ISR);
}

/**********************************************************************//**
 * @brief		Clear GPIO interrupt status
 * @param[in]	PCx	Pointer to selected PCU peripheral, should be: A,B,C,D
 * @return		None
 **********************************************************************/
void HAL_GPIO_EXTI_ClearPin(PCU_Type *PCx, uint32_t value)
{
//	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register	
	PCx->ISR = value;
//	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 	
}

/**********************************************************************//**
 * @brief 		Configure pin debounce
 * @param[in]	PCx	Pointer to selected PCU peripheral, should be: A,B,C,D
 * @param[in]	pin_no		Pin number, should be: 0..15
 * @param[in]	debounce debounce state, should be:
 * 					- DISABLE		:0
 * 					- ENABLE 		:1
 * @return		None
 **********************************************************************/
void HAL_GPIO_ConfigureDebounce (PCU_Type *PCx, uint8_t pin_no, FunctionalState debounce)
{
	uint32_t	reg_val; 

	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register
	reg_val = PCx->DER; 
	reg_val &= ~(1<<pin_no); 
	reg_val |= (debounce<<pin_no); 
	PCx->DER=reg_val; 
	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 		
}

/**********************************************************************//**
 * @brief		Set Value for bits that have output direction on GPIO port.
 * @param[in]	Px selected port, should be in range from PA to PD
 * @param[in]	bitValue Value that contains all bits on GPIO to set, should
 * 				be in range from 0 to 0xFFFF.
 * 				example: value 0x5 to set bit 0 and bit 1.
 * @return		None
 *
 * Note:
 * - For all bits that has been set as input direction, this function will
 * not effect.
 * - For all remaining bits that are not activated in bitValue (value '0')
 * will not be effected by this function.
 **********************************************************************/
void HAL_GPIO_SetPin(GPIO_Type *Px, uint16_t bitValue)
{
	Px->BSR = bitValue;
}


/**********************************************************************//**
 * @brief		Clear Value for bits that have output direction on GPIO port.
 * @param[in]	Px selected port, should be in range from PA to PD
 * @param[in]	bitValue Value that contains all bits on GPIO to clear, should
 * 				be in range from 0 to 0xFFFF.
 * 				example: value 0x5 to clear bit 0 and bit 1.
 * @return		None
 *
 * Note:
 * - For all bits that has been set as input direction, this function will
 * not effect.
 * - For all remaining bits that are not activated in bitValue (value '0')
 * will not be effected by this function.
 **********************************************************************/
void HAL_GPIO_ClearPin(GPIO_Type *Px, uint16_t bitValue)
{
	Px->BCR = bitValue;
}


/**********************************************************************//**
 * @brief		Read Current state on port pin that have input direction of GPIO
 * @param[in]	Px selected port, should be in range from PA to PD
 * @return		Current value of GPIO port.
 *
 * Note: Return value contain state of each port pin (bit) on that GPIO regardless
 * its direction is input or output.
 **********************************************************************/
uint16_t HAL_GPIO_ReadPin(GPIO_Type *Px)
{
	return Px->IDR;
}

/**********************************************************************//**
 * @brief		Write Value on port that have output direction of GPIO
 * @param[in]	Px selected port, should be in range from PA to PD
 * @return		None
 **********************************************************************/
void HAL_GPIO_WritePin(GPIO_Type *Px, uint16_t Value)
{
	Px->ODR = Value;
}

/* --------------------------------- End Of File ------------------------------ */
