/**********************************************************************//**
* @file		AC30M1x64_scu.c
* @brief	Contains all functions support for SCU firmware library on AC30M1x64
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


/* Includes ------------------------------------------------------------ */
#include "AC30M1x64_hal_scu.h"

/**********************************************************************//**
 * @brief 		Configure external sub clock  
 *             Before exteranl crystal enable setting, you have to set clock ports as SXIN,SXOUT in MR.
 * @param[in]	esosccon 	Function mode, should be:
 * 					- EXSOSC_STOP	   : 0
 * 					- EXSOSC_EN    	   : 2
 * 					- EXSOSC_EN_DIV2 : 3
 * @return		None
 **********************************************************************/
void HAL_SCU_LSE_ClockConfig(uint8_t esosccon)
{
	uint8_t reg_val8;
	
// enable external sub clock  
// Before exteranl sub crystal enable setting, you have to set sub clock Ports as SXIN,SXOUT in MR.
	reg_val8=SCU->CSCR;
	reg_val8&=~(3<<6); // 
	reg_val8|=(esosccon<<6); // 0,1:Stop External sub crystal, 2:Enable External sub crystal, 3:Enable External sub crystal divide by 2
	SCU->CSCR=reg_val8;	
}
 
/**********************************************************************//**
 * @brief 	  monitoring External sub oscillator  
 * @param  none
 * @return	status enumeration value:
 *              - ERROR
 *              - SUCCESS
 **********************************************************************/
Status HAL_SCU_LSE_ClockMonitoring(void)
{
	__IO uint32_t StartUpCounter = 0;
	__IO uint16_t status;
	uint32_t i;
		
	// monitoring External sub oscillator 
	SCU->CMR|=(1<<11); // 0:External sub oscillator monitoring disabled, 1:External sub oscillator monitoring enable
	
	/* Wait till SOSC is ready and if timeout is reached exit */
	do 
	{
		status = (SCU->CMR & SCU_CMR_SOSCSTS_Msk);
		StartUpCounter++;  
	} while((StartUpCounter != SOSC_STARTUP_TIMEOUT) && (status != SCU_CMR_SOSCSTS_Msk));	
	
	/* need more Wait for SOSC stable, check SXOUT pin oscillation  */	
	for (i=0;i<0x100;i++)
	{
		__NOP();
	}
	
	status = (SCU->CMR & SCU_CMR_SOSCSTS_Msk);
	if (status != SCU_CMR_SOSCSTS_Msk){
		return ERROR;
	}
	else {
		return SUCCESS;
	}
}

/**********************************************************************//**
 * @brief 		Configure internal ring osc clock  
 * @param[in]	ringosc Function mode, should be:
 * 					- INRINGOSC_STOP		   : 0
 * 					- INRINGOSC_EN    	     : 2
 * 					- INRINGOSC_EN_DIV2  : 3
 * @return		None
 **********************************************************************/
void HAL_SCU_LSI_ClockConfig(uint8_t ringosc)
{
	uint8_t reg_val8;
	
// enable internal clock  
	reg_val8=SCU->CSCR;
	reg_val8&=~(3<<4); // 
	reg_val8|=(ringosc<<4); // 0,1:Stop Internal ring oscillator, 2:Enable Internal ring oscillator, 3:Enable Internal ring divide by 2
	SCU->CSCR=reg_val8;	
}

/**********************************************************************//**
 * @brief 		Configure high speed internal osc clock  
 * @param[in]	hsiosc Function mode, should be:
 * 					- HSIOSC_STOP		   : 0
 * 					- HSIOSC_EN    	     : 2
 * 					- HSIOSC_EN_DIV2  : 3
 * @return		None
 **********************************************************************/
void HAL_SCU_HSI_ClockConfig(uint8_t hsiosc)
{
	uint8_t reg_val8;
	
// enable internal clock  
	reg_val8=SCU->CSCR;
	reg_val8&=~(3<<2); // 
	reg_val8|=(hsiosc<<2); // 0,1:Stop hsi oscillator, 2:Enable hsi oscillator, 3:Enable hsi divide by 2
	SCU->CSCR=reg_val8;	
}

/**********************************************************************//**
 * @brief 		Configure external clock  
 *             Before exteranl crystal enable setting, you have to set clock ports as XIN,XOUT in MR.
 * @param[in]	eosccon 	Function mode, should be:
 * 					- EXOSC_STOP	   : 0
 * 					- EXOSC_EN    	   : 2
 * 					- EXOSC_EN_DIV2 : 3
 * @return		None
 **********************************************************************/
void HAL_SCU_HSE_ClockConfig(uint8_t eosccon)
{
	uint8_t reg_val8;
	
// enable external clock  
// Before exteranl crystal enable setting, you have to set clock ports as XIN,XOUT in MR.
	reg_val8=SCU->CSCR;
	reg_val8&=~(3<<0); // 
	reg_val8|=(eosccon<<0); // 0,1:Stop External crystal oscillator, 2:Enable External crystal oscillator, 3:Enable External crystal divide by 2
	SCU->CSCR=reg_val8;	
}

/**********************************************************************//**
 * @brief 	  monitoring External oscillator  
 * @param  none
 * @return	status enumeration value:
 *              - ERROR
 *              - SUCCESS
 **********************************************************************/
Status HAL_SCU_HSE_ClockMonitoring(void)
{
	__IO uint32_t StartUpCounter = 0;
	__IO uint16_t status;
	uint32_t i;

	// monitoring External oscillator 
	SCU->CMR|=(1<<3); // 0:External oscillator monitoring disabled, 1:External oscillator monitoring enable
	
	/* Wait till MOSC is ready and if timeout is reached exit */
	do 
	{
		status = (SCU->CMR & SCU_CMR_MOSCSTS_Msk);
		StartUpCounter++;  
	} while((StartUpCounter != MOSC_STARTUP_TIMEOUT) && (status != SCU_CMR_MOSCSTS_Msk));	
	
	/* need more Wait for MOSC stable, check XOUT pin oscillation  */	
	for (i=0;i<0x800;i++)
	{
		__NOP();
	}
	
	status = (SCU->CMR & SCU_CMR_MOSCSTS_Msk);
	if (status != SCU_CMR_MOSCSTS_Msk){
		return ERROR;
	}
	else {
		return SUCCESS;
	}
}

/**********************************************************************//**
 * @brief 		change system clock control   
 * @param[in]	sysclkcon mode, should be:
 * 					- SC_INRINGOSC		         : 0
 * 					- SC_EXSOSC  	               : 1 
 * 					- SC_HSIOSC  	               : 4
 * 					- SC_EXOSC              : 6
 * @return		None
 **********************************************************************/
void HAL_SCU_SystemClockChange(uint8_t sysclkcon)
{
	uint8_t reg_val8;
	
	reg_val8=SCU->SCCR;
	reg_val8&=~(7<<0); //
	reg_val8|=sysclkcon;  
	SCU->SCCR=reg_val8;	
}


/**********************************************************************//**
 * @brief 		Configure clock out  on PC9
 *       Before this function setting, you have to set PC9 as CLKO in PCMR.
 * @param[in]	divval(=CLKODIV) : 0~15  // CLKO=MCLK/(2*(CLKODIV+1))
 *                   ex) MCLK=20Mh, CLKODIV=4,    20Mhz/ (2*(4+1)) = 2Mhz
 * @param[in]	FunctionalState, should be:
 * 					- DISABLE		   : 0
 * 					- ENABLE    	   : 1
 * @return		None
 **********************************************************************/
void HAL_SCU_ClockOutput(uint8_t divval, FunctionalState endis)
{
	SCU->COR=(endis<<4)|(divval&0xf);      
}

//#define USED_CLOCKSRC     SC_INRINGOSC
//#define USED_CLOCKSRC     SC_EXSOSC
#define USED_CLOCKSRC     SC_HSIOSC
//#define USED_CLOCKSRC     SC_EXOSC
/**********************************************************************//**
 * @brief		Initialize default clock for AC30M1x64 board
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void HAL_SCU_ClockInit(void)
{
	uint32_t i;

//!! For clock change, enable all clock source. !!
	SCU->CSCR = 0xAA;
	
// CLKO function setting. check PORT setting (PC9).
//	HAL_SCU_ClockOutput(4,DISABLE);
	HAL_SCU_ClockOutput(4,ENABLE);

	
#if 0 // internal HIOSC self trim test code
	{
		uint32_t tempreg;	
		
		FM->MR = 0xa5;       // after changing 0xa5 -> 0x5a in FMMR reg, trim mode will be able to be set.
		FM->MR = 0x5a;       // enter trim mode 
		
		tempreg = SCU->HSIOSCTRIM;
		tempreg &= 0xffffff00;
		tempreg |= ((1<<7) |(0x00<<0)); 
		SCU->HSIOSCTRIM = tempreg;	
		FM->MR = 0;	      // exit trim mode--> normal mode	
		
		SCU->HSIOSCTRIM|=((uint32_t)1<<31);
		SCU->BISCCON=(400<<16)|(80);
	}
#endif
	
	
	SCU->CMR&=~(1<<7); //mclk monitoring disable
	
#if (USED_CLOCKSRC==SC_INRINGOSC) // 40kHz

	HAL_SCU_LSE_ClockConfig(INRINGOSC_EN);
	for (i=0;i<100;i++);
	
	HAL_SCU_SystemClockChange(USED_CLOCKSRC);
	
	SystemCoreClock=40000;
	SystemPeriClock=40000;	
//	SCU->CSCR = 0x20;
	
#elif (USED_CLOCKSRC==SC_EXSOSC) //32.768kHz
// Before using this routine, you must set PORTs as SXIN and SXOUT.
// external SOSC enable 	
	HAL_SCU_LSE_ClockConfig(EXOSC_EN); 
	for (i=0;i<100;i++);
	
// wait for SOSC stable	
	HAL_SCU_LSE_ClockMonitoring();

	HAL_SCU_SystemClockChange(USED_CLOCKSRC);
	
	SystemCoreClock=32768; //32768Hz
	SystemPeriClock=32768; //32768Hz	
	
//	SCU->CSCR = 0x80;	
	
#elif (USED_CLOCKSRC==SC_HSIOSC) // 40Mhz
	HAL_SCU_HSI_ClockConfig(HSIOSC_EN); 
//	HAL_SCU_HSI_ClockConfig(HSIOSC_EN_DIV2); 
	for (i=0;i<100;i++);
	
	HAL_SCU_SystemClockChange(USED_CLOCKSRC);
	
	SystemCoreClock=40000000; //40Mhz  <- HSIOSC_EN
	SystemPeriClock=40000000; //40Mhz	

//	SystemCoreClock=20000000; //20Mhz  <- HSIOSC_EN_DIV2
//	SystemPeriClock=20000000; //20Mhz	

//	SCU->CSCR = 0x08;
	
#elif (USED_CLOCKSRC==SC_EXOSC) //4Mhz~16Mhz
// Before using this routine, you must set PORTs as XIN and XOUT.
// external MOSC enable 	
	HAL_SCU_HSE_ClockConfig(EXOSC_EN); 
//	HAL_SCU_HSE_ClockConfig(EXOSC_EN_DIV2); 
	
	for (i=0;i<10;i++);

// wait for MOSC stable	
	HAL_SCU_HSE_ClockMonitoring();
	
	HAL_SCU_SystemClockChange(USED_CLOCKSRC);
	
//	SystemCoreClock=8000000; //xMhz
//	SystemPeriClock=8000000; //xMhz	

	SystemCoreClock=16000000; //xMhz
	SystemPeriClock=16000000; //xMhz	
	
//	SystemCoreClock=4000000; //xMhz
//	SystemPeriClock=4000000; //xMhz	

//	SCU->CSCR = 0x02;
#else 
	while(1) {}	
 
#endif

	SCU->CMR|=(1<<7); //mclk monitoring enable

// flash memory controller
	FM->MR = 0x81;       // after changing 0x81 -> 0x28 in FMMR reg, flash access timing will be able to be set.
	FM->MR = 0x28;       // enter flash access timing changing mode
//	FM->CFG = (0x7858<<16) | (0<<8);  //flash access cycles 	
//	FM->CFG = (0x7858<<16) | (1<<8);  //flash access cycles 	
	FM->CFG = (0x7858<<16) | (2<<8);  //flash access cycles 			
//	FM->CFG = (0x7858<<16) | (3<<8);  //flash access cycles 			
	                              // flash access time cannot overflow 20Mhz.
	                              // ex) if MCLK=40Mhz, 
	                              //       40/1 = 40 (can't set no wait)
	                              //       40/2 = 20 (1 wait is ok)
	                              // so, 1 wait is possible.
	FM->MR = 0;	      // exit flash access timing --> normal mode			
}

/**********************************************************************//**
 * @brief 		Configure peri clock setting
 * @param[in]	mccrnum: select REG number
 *                     - MCCR1 : 1, * - MCCR2 : 2
 *                     - MCCR3 : 3, * - MCCR4 : 4
 *                     - MCCR5 : 5, * - MCCR7 : 7
 * @param[in]	type : 
 *	                     SYSTICK_TYPE, PWM0_TYPE, 
 *						WDT_TYPE, TEXT_TYPE, PAD_TYPE, PBD_TYPE, PCD_TYPE,
 *						PDD_TYPE, UARTC_TYPE, DCC_TYPE	
 * @param[in]	clksrc : 
 *						- CLKSRC_LSI=0,
 *						- CLKSRC_MCLK=4,
 *						- CLKSRC_HSI=5,	
 *						- CLKSRC_EOSC=6
 *						- CLKSRC_SOSC=7	
 * @param[in]	clkdiv : 
 * @return		None
 **********************************************************************/
void HAL_SCU_MiscClockConfig(uint8_t mccrnum, uint8_t type, uint8_t clksrc, uint8_t clkdiv)
{
	uint32_t tmp;
	
	switch(mccrnum){
		case 1:
			tmp=SCU->MCCR1;
			if (type==SYSTICK_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			SCU->MCCR1=tmp;
			break;
			
		case 2:
			tmp=SCU->MCCR2;
			if (type==PWM0_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			SCU->MCCR2=tmp;			
			break;
			
		case 3:
			tmp=SCU->MCCR3;
			if (type==WDT_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			else if (type==TEXT_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}
			SCU->MCCR3=tmp;			
			break;
			
		case 4:
			tmp=SCU->MCCR4;
			if (type==PAD_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			else if (type==PBD_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}
			SCU->MCCR4=tmp;			
			break;
			
		case 5:
			tmp=SCU->MCCR5;
			if (type==PCD_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			else if (type==PDD_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}
			SCU->MCCR5=tmp;			
			break;
			
		case 7:
			tmp=SCU->MCCR7;
			if (type==UARTC_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}		
			if (type==ADCC_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}
			SCU->MCCR7=tmp;			
			break;			
	}
}

/**********************************************************************//**
 * @brief 		set LVD level   
 * @param[in]	bodlevel, should be:
 * 					- LVD_LEVEL_1_65 : 0
 * 					- LVD_LEVEL_2_5 : 1
 * 					- LVD_LEVEL_3_6 : 2
 * 					- LVD_LEVEL_4_2 : 3
 * @return		None
 **********************************************************************/
void SCU_SetLVDLevel(uint32_t lvdlevel)
{
	uint32_t tmp;
	
	tmp=SCU->LVDCON;
	tmp&=~(3<<8); //
	tmp|=(1<<15) | (lvdlevel <<8); 
	SCU->LVDCON=tmp;	
}

/**********************************************************************//**
 * @brief	 	enable/disable LVD
 * @param[in]	NewState
 * 					- ENABLE  	:Set LVD enable
 * 					- DISABLE 	:Disable LVD
 * @return 		None
 **********************************************************************/
void HAL_SCU_LVR_Cmd(FunctionalState NewState)
{
	if (NewState == ENABLE)
	{
		SCU->LVDCON |=  (1<<0);
	}
	else
	{
		SCU->LVDCON &= ~(1<<0);
	}
}

/**********************************************************************//**
 * @brief	 	enable/disable WakeUp Source
 * @param[in]	WakeUpSrc, oring is possible:
 * 					- WAKEUP_GPIOD :    (1UL<<11)
 * 					- WAKEUP_GPIOC :    (1UL<<10)
 * 					- WAKEUP_GPIOB :    (1UL<<9)
 * 					- WAKEUP_GPIOA :   (1UL<<8)
*						- WAKEUP_FRT     :   (1UL<<2)
 * 					- WAKEUP_WDT    :    (1UL<<1)
 * 					- WAKEUP_LVD    :    (1UL<<0)
 * @param[in]	NewState
 * 					- ENABLE  	:Set WakeUp Source enable
 * 					- DISABLE 	:Disable WakeUp Source
 * @return 		None
 **********************************************************************/
void HAL_SCU_WakeUpSRCCmd(uint32_t WakeUpSrc, FunctionalState NewState)
{
	if (NewState == ENABLE)
	{
		SCU->WUER |=  (WakeUpSrc);
	}
	else
	{
		SCU->WUER &= ~(WakeUpSrc);
	}
}

/*********************************************************************//**
 * @brief 		Get current value of WakeUpSRC Status.
 * @param[in]	None.
 * @return		Current value of WakeUpSRC Status register.
 *********************************************************************/
uint32_t HAL_SCU_GetWakeUpSRCStatus(void)
{
	return (SCU->WUSR);
}

/**********************************************************************//**
 * @brief	 	enable/disable WakeUp Source
 * @param[in]	WakeUpSrc
 * @return 		None
 **********************************************************************/
void SCU_ClearWakeUpSRC(uint32_t WakeUpStatus)
{
	SCU->WUSR = WakeUpStatus;
}

/*********************************************************************//**
 * @brief 		Get current value of NMI Status.
 * @param[in]	None.
 * @return		Current value of NMI Status register.
 *********************************************************************/
uint32_t HAL_SCU_GetNMIStatus(void)
{
	return (SCU->NMIR);
}

/**********************************************************************//**
 * @brief	 	enable/disable NMI Source
 * @param[in]	nmisrc
 * @return 		None
 **********************************************************************/
void HAL_SCU_SetNMI(uint32_t nmisrc)
{
	SCU->NMIR = (0xA32CUL<<16) | (nmisrc & 0x1F);
}


/* --------------------------------- End Of File ------------------------------ */
