/**********************************************************************//**
* @file		AC30M1x164_hal_timer.c
* @brief	Contains all functions support for Timer firmware library on AC30M1x164
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


/* Peripheral group -------------------------------------------------------- */

/* Includes --------------------------------------------------------------- */
#include "AC30M1x64_hal_scu.h"
#include "AC30M1x64_hal_timer.h"




/* Public Functions --------------------------------------------------------- */
/**********************************************************************//**
 * @brief 		Initial Timer/Counter device
 * 				 	Set Clock frequency for Timer
 * 					Set initial configuration for Timer
 * @param[in]	TIMERx  Timer selection, should be:
 * 					- T0	:TIMER0 peripheral
 * 					- T1	:TIMER1 peripheral
 * 					- T2	:TIMER2 peripheral
 * 					- T3	:TIMER3 peripheral
 * @param[in]	TimerCounterMode Timer counter mode, should be:
 * 					- PERIODIC_MODE			:Timer mode
 * 					- PWM_MODE	:Counter rising mode
 * 					- ONESHOT_MODE	:Counter falling mode
 * 					- CAPTURE_MODE		:Counter on both edges
 * @param[in]	TIMER_ConfigStruct pointer to TIM_TIMERCFG_Type
 * 				that contains the configuration information for the
 *                    specified Timer peripheral.
 * @return 		None
 **********************************************************************/
HAL_Status_Type HAL_TIMER_Init(TIMER_Type *TIMERx, TIMER_MODE_OPT TimerCounterMode, void  *TIMER_ConfigStruct)
{
	
	TIMER_PERIODICCFG_Type *pTimeCfg;
	TIMER_PWMCFG_Type *pPwmOneshotCfg;
	TIMER_CAPTURECFG_Type *pCaptureCfg;
	uint16_t reg_val16;
	
	//set power
	if (TIMERx== T0)
	{
		SCU->PER1&=~(1<<16);
		SCU->PCER1&=~(1<<16);
		
		/* Set up peripheral clock for TIMER0 module */
		SCU->PER1|=(1<<16);
		SCU->PCER1|=(1<<16);
	}
	else if (TIMERx== T1)
	{
		SCU->PER1&=~(1<<17);
		SCU->PCER1&=~(1<<17);
		
		/* Set up peripheral clock for TIMER1 module */		
		SCU->PER1|=(1<<17);
		SCU->PCER1|=(1<<17);
	}
	else if (TIMERx== T2)
	{
		SCU->PER1&=~(1<<18);
		SCU->PCER1&=~(1<<18);
		
		/* Set up peripheral clock for TIMER2 module */	
		SCU->PER1|=(1<<18);
		SCU->PCER1|=(1<<18);
	}
	else if (TIMERx== T3)
	{
		SCU->PER1&=~(1<<19);
		SCU->PCER1&=~(1<<19);
		
		/* Set up peripheral clock for TIMER3 module */			
		SCU->PER1|=(1<<19);
		SCU->PCER1|=(1<<19);
	}
	

	if (TimerCounterMode == PERIODIC_MODE)
	{
		pTimeCfg = (TIMER_PERIODICCFG_Type *)TIMER_ConfigStruct;
		reg_val16 = 0
			|TimerCounterMode
//			|(1<<12)		
			|TIMER_CR1_CKSEL_SET(pTimeCfg->CkSel) 
			|TIMER_CR1_STARTLVL_SET(pTimeCfg->StartLevel)
			|TIMER_CR1_ADCTRGEN_SET(pTimeCfg->AdcTrgEn)
			;
		TIMERx->CR1 = reg_val16;
		TIMERx->PRS = (pTimeCfg->Prescaler & TIMER_PRS_MASK);
		TIMERx->GRA = pTimeCfg->GRA;
		TIMERx->GRB = pTimeCfg->GRB;		
		TIMERx->CNT = pTimeCfg->Cnt;
	}
	else if ((TimerCounterMode == PWM_MODE) || (TimerCounterMode == ONESHOT_MODE))
	{
		pPwmOneshotCfg = (TIMER_PWMCFG_Type *)TIMER_ConfigStruct;
		reg_val16 = 0
			|TimerCounterMode
			|TIMER_CR1_CKSEL_SET(pPwmOneshotCfg->CkSel) 
			|TIMER_CR1_STARTLVL_SET(pPwmOneshotCfg->StartLevel)
			|TIMER_CR1_ADCTRGEN_SET(pPwmOneshotCfg->AdcTrgEn)
			;
		TIMERx->CR1 = reg_val16;
		TIMERx->PRS = (pPwmOneshotCfg->Prescaler & TIMER_PRS_MASK);
		TIMERx->GRA = pPwmOneshotCfg->GRA;		
		TIMERx->GRB = pPwmOneshotCfg->GRB;	
		TIMERx->CNT = pPwmOneshotCfg->Cnt;		
	}
	else if (TimerCounterMode == CAPTURE_MODE)
	{
		pCaptureCfg = (TIMER_CAPTURECFG_Type *)TIMER_ConfigStruct;
		reg_val16 = 0
			|TimerCounterMode
			|TIMER_CR1_CLRMD_SET(pCaptureCfg->ClrMode)		
			|TIMER_CR1_CKSEL_SET(pCaptureCfg->CkSel) 
			|TIMER_CR1_ADCTRGEN_SET(pCaptureCfg->AdcTrgEn)
			;
		TIMERx->CR1 = reg_val16;
		TIMERx->PRS = (pCaptureCfg->Prescaler & TIMER_PRS_MASK);
	}
	TIMERx->CR2 = 0x2; //  timer counter clear and disable timer 	
	
	return HAL_OK;
}

/**********************************************************************//**
 * @brief 		Close Timer/Counter device
 * @param[in]	TIMx  Pointer to timer device, should be:
 * 					- T0	:TIMER0 peripheral
 * 					- T1	:TIMER1 peripheral
 * 					- T2	:TIMER2 peripheral
 * 					- T3	:TIMER3 peripheral
 * @return 		None
 **********************************************************************/
HAL_Status_Type HAL_TIMER_DeInit (TIMER_Type *TIMERx)
{
	// Disable timer/counter
	TIMERx->CR2 = 0x00;
	return HAL_OK;
}

/**********************************************************************//**
 * @brief 		Clear Timer Status
 * @param[in]	TIMERx  Timer selection, should be:
 * 					- T0	:TIMER0 peripheral
 * 					- T1	:TIMER1 peripheral
 * 					- T2	:TIMER2 peripheral
 * 					- T3	:TIMER3 peripheral
 * @param[in]	status clear value
 * @return 		None
 **********************************************************************/
HAL_Status_Type HAL_TIMER_ClearStatus(TIMER_Type *TIMERx, uint8_t value)
{
	TIMERx->SR = value;
	return HAL_OK;
}

/**********************************************************************//**
 * @brief 		Get Timer Status
 * @param[in]	TIMERx  Timer selection, should be:
 * 					- T0	:TIMER0 peripheral
 * 					- T1	:TIMER1 peripheral
 * 					- T2	:TIMER2 peripheral
 * 					- T3	:TIMER3 peripheral
 * @return 		Value of status register
 **********************************************************************/
uint8_t TIMER_GetStatus(TIMER_Type *TIMERx)
{
	return (TIMERx->SR);
}

/**********************************************************************//**
 * @brief	 	Start/Stop Timer/Counter device
 * @param[in]	TIMERx Pointer to timer device, should be:
 * 					- T0	:TIMER0 peripheral
 * 					- T1	:TIMER1 peripheral
 * 					- T2	:TIMER2 peripheral
 * 					- T3	:TIMER3 peripheral
 * @param[in]	NewState
 * 					- ENABLE  	:Set timer enable
 * 					- DISABLE 	:Disable timer
 * @return 		None
 **********************************************************************/
HAL_Status_Type HAL_TIMER_Cmd(TIMER_Type *TIMERx, FunctionalState NewState)
{
	if (NewState == ENABLE) {
		TIMERx->CR2 =  TIMER_ENABLE | TIMER_CLEAR;	
	}
	else {
		TIMERx->CR2 &= ~TIMER_ENABLE;
	}
	return HAL_OK;
}

/**********************************************************************//**
 * @brief 		Clear Timer/Counter device,
 * @param[in]	TIMERx Pointer to timer device, should be:
 * 					- T0	:TIMER0 peripheral
 * 					- T1	:TIMER1 peripheral
 * 					- T2	:TIMER2 peripheral
 * 					- T3	:TIMER3 peripheral
 * @return 		None
 **********************************************************************/
HAL_Status_Type HAL_TIMER_ClearCounter(TIMER_Type *TIMERx)
{
	TIMERx->CR2 |= TIMER_CLEAR;
	return HAL_OK;
}

/**********************************************************************//**
 * @brief 		Update value
 * @param[in]	TIMERx Pointer to timer device, should be:
 * 					- T0	:TIMER0 peripheral
 * 					- T1	:TIMER1 peripheral
 * 					- T2	:TIMER2 peripheral
 * 					- T3	:TIMER3 peripheral
 * @param[in]	CountCh, should be: 0=GRA,1=GRB,2=CNT
 * @param[in]	MatchValue		updated match value
 * @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_TIMER_UpdateCountValue(TIMER_Type *TIMERx, uint8_t CountCh, uint16_t Value)
{
	switch(CountCh) {
	case 0:
		TIMERx->GRA = Value;
		break;
	case 1:
		TIMERx->GRB = Value;
		break;
	case 2:
		TIMERx->CNT = Value;
		break;	
	}
	return HAL_OK;
}

/*********************************************************************//**
 * @brief 		Read value of capture register in timer/counter device
 * @param[in]	TIMERx Pointer to timer device, should be:
 * 					- T0	:TIMER0 peripheral
 * 					- T1	:TIMER1 peripheral
 * 					- T2	:TIMER2 peripheral
 * 					- T3	:TIMER3 peripheral
 * @param[in]	CountCh, should be: 0=GRA,1=GRB,2=CNT
 * @return 		Value of count register
 **********************************************************************/
uint16_t HAL_TIMER_GetCountValue(TIMER_Type *TIMERx, uint8_t CountCh)
{
	switch(CountCh){
		case 0: 	return TIMERx->GRA ;
		case 1:	return TIMERx->GRB ;
		case 2:	return TIMERx->CNT;
	}
	return 0;
}

/********************************************************************//**
 * @brief 		Enable or disable TIMER interrupt.
 * @param[in]	TIMERx Pointer to timer device, should be:
 * 					- T0	:TIMER0 peripheral
 * 					- T1	:TIMER1 peripheral
 * 					- T2	:TIMER2 peripheral
 * 					- T3	:TIMER3 peripheral
 * @param[in]	UARTIntCfg	Specifies the interrupt flag,
 * 				should be one of the following:
 *						- TIMER_INTCFG_OVIE : OVIE Interrupt enable
 *						- TIMER_INTCFG_MBIE : MBIE Interrupt enable
 *						- TIMER_INTCFG_MAIE : MAIE interrupt enable
 *						- TIMER_INTCFG_QRIE : QRIE interrupt 
 *						- TIMER_INTCFG_QDIRCHIE: QDIRCHIE interrupt 
*						- TIMER_INTCFG_QERRIE:  QERRIE interrupt
 * @param[in]	NewState New state of specified UART interrupt type,
 * 				should be:
 * 					- ENALBE	:Enable this interrupt type.
 * 					- DISALBE	:Disable this interrupt type.
 * @return 		None
 *********************************************************************/
HAL_Status_Type HAL_TIMER_ConfigInterrupt(TIMER_Type *TIMERx, TIMER_INT_Type TIMERIntCfg, FunctionalState NewState)
{
	uint8_t tmp;

	switch(TIMERIntCfg){
		case TIMER_INTCFG_OVIE:
			tmp = TIMER_IER_OVIE;
			break;
		case TIMER_INTCFG_MBIE:
			tmp = TIMER_IER_MBIE;
			break;
		case TIMER_INTCFG_MAIE:
			tmp = TIMER_IER_MAIE;
			break;
		case TIMER_INTCFG_QRIE:
			tmp = TIMER_IER_QRIE;
			break;
		case TIMER_INTCFG_QDIRCHIE:
			tmp = TIMER_IER_QDIRCHIE;
			break;		
		case TIMER_INTCFG_QERRIE:
			tmp = TIMER_IER_QERRIE;
			break;			
	}

	if (NewState == ENABLE)
	{
		TIMERx->IER |= tmp;
	}
	else
	{
		TIMERx->IER &= (~tmp) & TIMER_IER_BITMASK;
	}
	return HAL_OK;
}




/* --------------------------------- End Of File ------------------------------ */
