#include "AC30M1x64_hal_pcu.h"
#include "AC30M1x64_hal_scu.h"
#include "AC30M1x64_hal_uart.h"
#include "AC30M1x64_hal_libcfg.h"


/* buffer size definition */
#define UART_RING_BUFSIZE 256

/* Buf mask */
#define __BUF_MASK (UART_RING_BUFSIZE-1)
/* Check buf is full or not */
#define __BUF_IS_FULL(head, tail) ((tail&__BUF_MASK)==((head+1)&__BUF_MASK))
/* Check buf will be full in next receiving or not */
#define __BUF_WILL_FULL(head, tail) ((tail&__BUF_MASK)==((head+2)&__BUF_MASK))
/* Check buf is empty */
#define __BUF_IS_EMPTY(head, tail) ((head&__BUF_MASK)==(tail&__BUF_MASK))
/* Reset buf */
#define __BUF_INCR(bufidx)	(bufidx=(bufidx+1)&__BUF_MASK)


typedef struct
{
    __IO uint32_t tx_head;                /*!< UART Tx ring buffer head index */
    __IO uint32_t tx_tail;                /*!< UART Tx ring buffer tail index */
    __IO uint32_t rx_head;                /*!< UART Rx ring buffer head index */
    __IO uint32_t rx_tail;                /*!< UART Rx ring buffer tail index */
    __IO uint8_t  tx[UART_RING_BUFSIZE];  /*!< UART Tx data ring buffer */
    __IO uint8_t  rx[UART_RING_BUFSIZE];  /*!< UART Rx data ring buffer */
} UART_RING_BUFFER_T;


// UART Ring buffer
UART_RING_BUFFER_T rb;

// Current Tx Interrupt enable state
__IO FlagStatus TxIntStat;

/* Interrupt service routines */
void UART0_IRQHandler(void);
void UART_IntErr(uint8_t bLSErrType);
void UART_IntTransmit(void);
void UART_IntReceive(void);

uint32_t UARTReceive(UART_Type *UARTx, uint8_t *rxbuf, uint8_t buflen);
uint32_t UARTSend(UART_Type *UARTx, uint8_t *txbuf, uint8_t buflen);

// system default I/O setting
void PCU_Init(void);

/*----------------- INTERRUPT SERVICE ROUTINES --------------------------*/
/*********************************************************************//**
 * @brief		UART0 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void UART0_IRQHandler(void)
{
	uint32_t intsrc, tmp, tmp1;

	/* Determine the interrupt source */
	intsrc = U0->IIR;
	tmp = intsrc & UART_IIR_INTID_MASK;

	// Receive Line Status
	if (tmp == UART_IIR_INTID_RLS){
		// Check line status
		tmp1 = U0 ->LSR;
		// Mask out the Receive Ready and Transmit Holding empty status
		tmp1 &= (UART_LSR_OE | UART_LSR_PE | UART_LSR_FE | UART_LSR_BI);
		// If any error exist
		if (tmp1) {
				UART_IntErr(tmp1);
		}
	}

	// Receive Data Available or Character time-out
	if (tmp == UART_IIR_INTID_RDA){
			UART_IntReceive();
	}

	// Transmit Holding Empty
	if (tmp == UART_IIR_INTID_THRE){
			UART_IntTransmit();
	}
}

/********************************************************************//**
 * @brief 		UART receive function (ring buffer used)
 * @param[in]	None
 * @return 		None
 *********************************************************************/
void UART_IntReceive(void)
{
	uint8_t tmpc;
	uint32_t rLen;

	while(1){
		// Call UART read function in UART driver
		rLen = HAL_UART_Receive((UART_Type *)U0, &tmpc, 1, NONE_BLOCKING);
		// If data received
		if (rLen){
			/* Check if buffer is more space
			 * If no more space, remaining character will be trimmed out
			 */
			if (!__BUF_IS_FULL(rb.rx_head,rb.rx_tail)){
				rb.rx[rb.rx_head] = tmpc;
				__BUF_INCR(rb.rx_head);
			}
		}
		// no more data
		else {
			break;
		}
	}
}

/********************************************************************//**
 * @brief 		UART transmit function (ring buffer used)
 * @param[in]	None
 * @return 		None
 *********************************************************************/
void UART_IntTransmit(void)
{
    // Disable THRE interrupt
    HAL_UART_ConfigInterrupt((UART_Type *)U0, UART_INTCFG_THRE, DISABLE);

	/* Wait until THR empty */
    while (HAL_UART_CheckBusy((UART_Type *)U0) == SET);

	while (!__BUF_IS_EMPTY(rb.tx_head,rb.tx_tail))
    {
        /* Move a piece of data into the transmit FIFO */
    	if (HAL_UART_Transmit((UART_Type *)U0, (uint8_t *)&rb.tx[rb.tx_tail], 1, NONE_BLOCKING)){
        /* Update transmit ring FIFO tail pointer */
        __BUF_INCR(rb.tx_tail);
    	} else {
    		break;
    	}
    }

    /* If there is no more data to send, disable the transmit
       interrupt - else enable it or keep it enabled */
	if (__BUF_IS_EMPTY(rb.tx_head, rb.tx_tail)) {
    	HAL_UART_ConfigInterrupt((UART_Type *)U0, UART_INTCFG_THRE, DISABLE);
    	// Reset Tx Interrupt state
    	TxIntStat = RESET;
    }
    else{
      	// Set Tx Interrupt state
		TxIntStat = SET;
    	HAL_UART_ConfigInterrupt((UART_Type *)U0, UART_INTCFG_THRE, ENABLE);
    }
}


/*********************************************************************//**
 * @brief		UART Line Status Error
 * @param[in]	bLSErrType	UART Line Status Error Type
 * @return		None
 **********************************************************************/
void UART_IntErr(uint8_t bLSErrType)
{
	uint8_t test;
	// Loop forever
	while (1){
		// For testing purpose
		test = bLSErrType;
		test = test;
	}
}

/*********************************************************************//**
 * @brief		UART transmit function for interrupt mode (using ring buffers)
 * @param[in]	UARTPort	Selected UART peripheral used to send data,
 * 				should be UART0
 * @param[out]	txbuf Pointer to Transmit buffer
 * @param[in]	buflen Length of Transmit buffer
 * @return 		Number of bytes actually sent to the ring buffer
 **********************************************************************/
uint32_t UARTSend(UART_Type *UARTx, uint8_t *txbuf, uint8_t buflen)
{
    uint8_t *data = (uint8_t *) txbuf;
    uint32_t bytes = 0;

	/* Temporarily lock out UART transmit interrupts during this
	   read so the UART transmit interrupt won't cause problems
	   with the index values */
    HAL_UART_ConfigInterrupt(UARTx, UART_INTCFG_THRE, DISABLE);

	/* Loop until transmit run buffer is full or until n_bytes
	   expires */
	while ((buflen > 0) && (!__BUF_IS_FULL(rb.tx_head, rb.tx_tail)))
	{
		/* Write data from buffer into ring buffer */
		rb.tx[rb.tx_head] = *data;
		data++;

		/* Increment head pointer */
		__BUF_INCR(rb.tx_head);

		/* Increment data count and decrement buffer size count */
		bytes++;
		buflen--;
	}

	/*
	 * Check if current Tx interrupt enable is reset,
	 * that means the Tx interrupt must be re-enabled
	 * due to call UART_IntTransmit() function to trigger
	 * this interrupt type
	 */
	if (TxIntStat == RESET) {
		UART_IntTransmit();
	}
	/*
	 * Otherwise, re-enables Tx Interrupt
	 */
	else {
		HAL_UART_ConfigInterrupt(UARTx, UART_INTCFG_THRE, ENABLE);
	}

    return bytes;
}


/*********************************************************************//**
 * @brief		UART read function for interrupt mode (using ring buffers)
 * @param[in]	UARTPort	Selected UART peripheral used to send data,
 * 				should be UART0
 * @param[out]	rxbuf Pointer to Received buffer
 * @param[in]	buflen Length of Received buffer
 * @return 		Number of bytes actually read from the ring buffer
 **********************************************************************/
uint32_t UARTReceive(UART_Type *UARTx, uint8_t *rxbuf, uint8_t buflen)
{
    uint8_t *data = (uint8_t *) rxbuf;
    uint32_t bytes = 0;

	/* Temporarily lock out UART receive interrupts during this
	   read so the UART receive interrupt won't cause problems
	   with the index values */
	HAL_UART_ConfigInterrupt(UARTx, UART_INTCFG_RBR, DISABLE);

	/* Loop until receive buffer ring is empty or
		until max_bytes expires */
	while ((buflen > 0) && (!(__BUF_IS_EMPTY(rb.rx_head, rb.rx_tail))))
	{
		/* Read data from ring buffer into user buffer */
		*data = rb.rx[rb.rx_tail];
		data++;

		/* Update tail pointer */
		__BUF_INCR(rb.rx_tail);

		/* Increment data count and decrement buffer size count */
		bytes++;
		buflen--;
	}

	/* Re-enable UART interrupts */
	HAL_UART_ConfigInterrupt(UARTx, UART_INTCFG_RBR, ENABLE);

    return bytes;
}

void mainloop(void)
{
	UART_CFG_Type UARTConfigStruct;	
	uint32_t idx, len;
	uint8_t buffer[10];


//UART0 pin config setting in PCU
	HAL_GPIO_ConfigFunction(PCC, 14, PC14_MUX_RXD0);
	HAL_GPIO_ConfigOutput(PCC, 14, INPUT);
	HAL_GPIO_ConfigPullup(PCC, 14, ENABLE);
	
	HAL_GPIO_ConfigFunction(PCC, 15, PC15_MUX_TXD0);
	HAL_GPIO_ConfigOutput(PCC, 15, PUSH_PULL_OUTPUT);	
	HAL_GPIO_ConfigPullup(PCC, 15, ENABLE);	


	// default: 38400-8-N-1	
	HAL_UART_ConfigStructInit(&UARTConfigStruct);
	HAL_UART_Init(U0, &UARTConfigStruct);

	    /* Enable UART Rx interrupt */
	HAL_UART_ConfigInterrupt((UART_Type *)U0, UART_INTCFG_RBR, ENABLE);
	/* Enable UART line status interrupt */
	HAL_UART_ConfigInterrupt((UART_Type *)U0, UART_INTCFG_RLS, ENABLE);
	
	TxIntStat = RESET;
	
	// Reset ring buf head and tail idx
	rb.rx_head=0;
	rb.rx_tail=0;
	rb.tx_head=0;
	rb.tx_tail=0;

    /* preemption = 1, sub-priority = 1 */
    NVIC_SetPriority(UART0_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for UART0 channel */
    NVIC_EnableIRQ(UART0_IRQn);
	
	__enable_irq();
	UARTSend(U0,(uint8_t *)"Interrupt test \n\r",17);	
	while(1){
       len = 0;
        while (len == 0)
        {
            len = UARTReceive((UART_Type *)U0, buffer, sizeof(buffer));
        }

        /* Got some data */
        idx = 0;
        while (idx < len)
        {
            if (buffer[idx] == 0x0d)
            {
                UARTSend((UART_Type *)U0, (uint8_t *)"\n\r", 2);
            }
            else
            {
                /* Echo it back */
            	UARTSend((UART_Type *)U0, &buffer[idx], 1);
            }
            idx++;
        }
	}
}

int main(void)
{
	SystemInit();
	PCU_Init(); // all port control setting  !!You have to set PORT function!!
	HAL_SCU_ClockInit(); // MCLK

	mainloop();
	
	return (0);
}


void PCU_Init(void)
{
//Peripheral Enable Register 1  0:Disable, 1:Enable	
	SCU->PER1=SCU->PER1  
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 		
//Peripheral Clock Enable Register 1 0:Disable, 1:Enable	
	SCU->PCER1=SCU->PCER1
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 	
	
 	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register

	PCA->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0<<30)   // 0:PA15,          1:T3IO                                               
	| (0<<28)   // 0:PA14,          1:T2IO                                                
	| (0<<26)   // 0:PA13,          1:T1IO                                                
	| (0<<24)   // 0:PA12,          1:T0IO                                     
	| (0<<22)   // 0:PA11,                                                          3:AN11                                                                     
	| (0<<20)   // 0:PA10,                                                          3:AN10
	| (0<<18)   // 0:PA9,            1:T3IO,                  2:T1IO,        3:AN9
	| (0<<16)   // 0:PA8,            1:T2IO,                  2:T0IO,        3:AN8
	| (0<<14)   // 0:PA7,            1:T1IO,                                     3:AN7
	| (0<<12)   // 0:PA6,            1:T0IO,                                     3:AN6
	| (0<<10)   // 0:PA5,                                                            3:AN5
	| (0<<8)   // 0:PA4,                                                              3:AN4
	| (0<<6)   // 0:PA3,              1:SCK0                 2:STBO,       3:AN3
	| (0<<4)   // 0:PA2,              1:SS0                   2:WDTO,      3:AN2	
	| (0<<2)   // 0:PA1,              1:T3IO,                                     3:AN1
	| (0<<0)   // 0:PA0,              1:T2IO,                                     3:AN0			
		; 
	PCA->CR = 0 // Input, output, open drain, analog
	| (0<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0<<28)   // P14
	| (0<<26)   // P13
	| (0<<24)   // P12
	| (0<<22)   // P11
	| (0<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
		; 	
	PCA->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 		
	
	
	PCB->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0<<14)   // 0:PB7,          1:OVIN0
	| (0<<12)   // 0:PB6,          1:PRTIN0
	| (0<<10)   // 0:PB5,          1:MP0WL
	| (0<<8)   // 0:PB4,          1:MP0WH          
	| (0<<6)   // 0:PB3,          1:MP0VL,          2:MISO0
	| (0<<4)   // 0:PB2,          1:MP0VH,          2:MOSI0
	| (0<<2)   // 0:PB1,          1:MP0UL,          2:SCK0
	| (0<<0)   // 0:PB0,          1:MP0UH,          2:SS0 
		;	
	PCB->CR = 0   // Input, output, open drain, analog
	| (0<<14)   // P7   0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
		; 	
	PCB->PCR = 0 // pullup (0,1)
	| (0<<7)   // P7  0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 
	
	PCC->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (1<<30)   // 0:PC15,          1:TXD0
	| (1<<28)   // 0:PC14,          1:RXD0
	| (3<<26)   // 0:PC13,        1:T2IO,                                           3:XOUT
	| (3<<24)   // 0:PC12,        1:T3IO,                                           3:XIN
	| (0<<22)   // 0:PC11,                                      2:T0IO
	| (1<<20)   // 0:PC10,        1:nRESET
	| (1<<18)   // 0:PC9,          1:CLKO
	| (0<<16)   // 0:PC8,          1:SDA0
	| (0<<14)   // 0:PC7,          1:SCL0,                  2:T3IO
	| (0<<12)   // 0:PC6,          1:TXD1,                   2:T2IO
	| (0<<10)   // 0:PC5,          1:RXD1,                  2:T1IO
	| (0<<8)   // 0:PC4,          1:nTRST,                  2:T0IO
	| (0<<6)   // 0:PC3
	| (0<<4)   // 0:PC2
	| (1<<2)   // 0:PC1,          1:SWDIO,                  2:TXD1
	| (1<<0)   // 0:PC0,          1:SWCLK,                 2:RXD1
		;		
	PCC->CR = 0   // Input, output, open drain, analog
	| (0<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (2<<28)   // P14
	| (3<<26)   // P13  if XOUT, it must be Analog 
	| (3<<24)   // P12  if XIN, it must be Analog
	| (0<<22)   // P11
	| (2<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (2<<2)   // P1
	| (2<<0)   // P0	
		; 	
	PCC->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 

	PCD->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0<<6)   // 0:PD3,          1:MISO0,           2:SDA0,          3:SXIN
	| (0<<4)   // 0:PD2,          1:MOSI0,           2:SCL0,          3:SXOUT
	| (0<<2)   // 0:PD1,          1:SCK0,           2:RXD0
	| (0<<0)   // 0:PD0,          1:SS0,             2:TXD0
		;		
	PCD->CR = 0   // Input, output, open drain, analog
	| (0<<6)   // P3  0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
		; 	
	PCD->PCR = 0 // pullup (0,1)
	| (0<<3)   // P3 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 
	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 	
}
